package cn.x47.service;


import cn.x47.handle.RIPPacketDecoder;
import cn.x47.handle.RIPServerHandler;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.DatagramChannel;
import io.netty.channel.socket.nio.NioDatagramChannel;

import java.net.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

public class RIPServer {

    public void start() throws InterruptedException, UnknownHostException, SocketException {
        NioEventLoopGroup group = new NioEventLoopGroup();
        try {
            Bootstrap b = new Bootstrap();
            b.group(group)
                    .channel(NioDatagramChannel.class)
                    .option(ChannelOption.SO_BROADCAST, true)
                    .option(ChannelOption.IP_MULTICAST_LOOP_DISABLED, true)
                    .handler(new ChannelInitializer<DatagramChannel>() {
                        @Override
                        protected void initChannel(DatagramChannel ch) throws Exception {

                            ch.pipeline().addLast(new RIPPacketDecoder(), new RIPServerHandler());
                        }
                    });

            ChannelFuture future = b.bind(520).sync();
            // 获取并加入组播组
            NioDatagramChannel channel = (NioDatagramChannel) future.channel();
            InetAddress multicastAddress = InetAddress.getByName("224.0.0.9"); // 替换为你的组播地址

            // 自动获取网络接口并加入组播组
            List<NetworkInterface> multicastInterfaces = getMulticastInterfaces();
            for (NetworkInterface networkInterface : multicastInterfaces) {
                channel.joinGroup(new InetSocketAddress(multicastAddress, 520), networkInterface).sync();
                System.out.println("已在接口 " + networkInterface.getName() + " 上加入组播组");
            }
            System.out.println("监听器已启动，正在监听端口：" + 520);
            future.channel().closeFuture().await();
        } finally {
            group.shutdownGracefully();
        }
    }

    private List<NetworkInterface> getMulticastInterfaces() throws SocketException {
        List<NetworkInterface> interfaces = new ArrayList<>();
        Enumeration<NetworkInterface> networkInterfaces = NetworkInterface.getNetworkInterfaces();
        while (networkInterfaces.hasMoreElements()) {
            NetworkInterface networkInterface = networkInterfaces.nextElement();
            if (networkInterface.isUp() && !networkInterface.isLoopback() && networkInterface.supportsMulticast()) {
                // 检查是否有 IPv4 地址
                Enumeration<InetAddress> addresses = networkInterface.getInetAddresses();
                boolean hasIPv4 = false;
                while (addresses.hasMoreElements()) {
                    InetAddress address = addresses.nextElement();
                    if (address instanceof Inet4Address) {
                        hasIPv4 = true;
                        break;
                    }
                }
                if (hasIPv4) {
                    interfaces.add(networkInterface);
                }
            }
        }
        return interfaces;
    }
}
